from typing import Any, Optional

import orjson
from rest_framework.renderers import BaseRenderer
from rest_framework.settings import api_settings
from rest_framework.utils import encoders


class ORJSONRenderer(BaseRenderer):
    """
    Renderer which serializes to JSON.
    Rendering with orjson.

    >>> ORJSONRenderer().render(None) == b""
    True
    >>> ORJSONRenderer().render({}) == b"{}"
    True
    >>> api_settings.user_settings["ORJSON_RENDERER_OPTION"] = orjson.OPT_APPEND_NEWLINE
    >>> ORJSONRenderer().render({}) == b"{}\\n"
    True
    """  # noqa: E501

    media_type = "application/json"
    format = "json"
    encoder_class = encoders.JSONEncoder

    # We don't set a charset because JSON is a binary encoding,
    # that can be encoded as utf-8, utf-16 or utf-32.
    # See: https://www.ietf.org/rfc/rfc4627.txt
    # Also: http://lucumr.pocoo.org/2013/7/19/application-mimetypes-and-encodings/
    charset = None

    def render(
        self,
        data: Any,
        accepted_media_type: Optional[str] = None,
        renderer_context: Optional[dict] = None,
    ) -> bytes:
        """
        Render `data` into JSON, returning a bytestring.
        """
        if data is None:
            return b""

        option = api_settings.user_settings.get("ORJSON_RENDERER_OPTION")

        return orjson.dumps(
            data,
            default=self.encoder_class().default,
            option=option,
        )



from io import BytesIO
from typing import Optional

import orjson
from rest_framework import renderers
from rest_framework.parsers import BaseParser, ParseError


class ORJSONParser(BaseParser):
    """
    Parses JSON-serialized data.
    Parsing with orjson.
    >>> ORJSONParser().parse(BytesIO(b"{}")) == {}
    True
    >>> ORJSONParser().parse(BytesIO(b"wrong"))
    Traceback (most recent call last):
        ...
    rest_framework.exceptions.ParseError: JSON parse error - unexpected character: line 1 column 1 (char 0)
    """  # noqa: E501

    media_type = "application/json"
    renderer_class = renderers.JSONRenderer

    def parse(
        self,
        stream: BytesIO,
        media_type: Optional[str] = None,
        parser_context: Optional[dict] = None,
    ):
        """
        Parses the incoming bytestream as JSON and returns the resulting data.
        """
        try:
            return orjson.loads(stream.read())
        except ValueError as exc:
            raise ParseError(f"JSON parse error - {exc}") from exc
